# This file is automatically generated by pyo3_stub_gen
# ruff: noqa: E501, F401


class PyHexmoveImuData:
    x_angle: float
    y_angle: float
    z_angle: float
    x_velocity: float
    y_velocity: float
    z_velocity: float

class PyHexmoveImuReader:
    def __new__(cls,interface:str, serial_number:int, model:int): ...
    def get_data(self) -> PyHexmoveImuData:
        ...

    def stop(self) -> None:
        ...


